from __future__ import annotations

from typing import Any, Dict, Optional

import httpx

from .doc import DocAPI
from .errors import RustKissVDBError
from .sql import SqlAPI
from .state import StateAPI
from .stream import StreamAPI
from .vector import VectorAPI


class Client:
    def __init__(
        self,
        base_url: str,
        api_key: Optional[str] = None,
        timeout: float = 30.0,
    ) -> None:
        if not base_url:
            raise ValueError("base_url requerido")

        self._base_url = base_url.rstrip("/")

        headers: Dict[str, str] = {}
        if api_key:
            headers["Authorization"] = f"Bearer {api_key}"

        self._http = httpx.Client(
            base_url=self._base_url,
            timeout=timeout,
            headers=headers,
        )

        # Sub-APIs (no import circular si ellos NO importan Client en runtime)
        self.state = StateAPI(self)
        self.vector = VectorAPI(self)
        self.doc = DocAPI(self)
        self.sql = SqlAPI(self)
        self.stream = StreamAPI(self)

    def close(self) -> None:
        self._http.close()

    def __enter__(self) -> "Client":
        return self

    def __exit__(self, exc_type, exc, tb) -> None:
        self.close()

    def request(
        self,
        method: str,
        path: str,
        *,
        params: Optional[Dict[str, Any]] = None,
        json: Optional[Dict[str, Any]] = None,
    ) -> Any:
        resp = self._http.request(method, path, params=params, json=json)

        if resp.status_code >= 400:
            raise RustKissVDBError(self._error_message(resp))

        ctype = resp.headers.get("content-type", "")
        if ctype.startswith("application/json"):
            return resp.json()
        return resp.text

    def stream_request(
        self,
        method: str,
        path: str,
        *,
        params: Optional[Dict[str, Any]] = None,
    ) -> httpx.Response:
        req = self._http.build_request(method, path, params=params)
        return self._http.send(req, stream=True)

    @staticmethod
    def _error_message(resp: httpx.Response) -> str:
        try:
            data = resp.json()
            err = data.get("error") or "error"
            msg = data.get("message") or resp.text
            return f"{err} - {msg}"
        except Exception:
            return resp.text



from __future__ import annotations

import json
from typing import TYPE_CHECKING, Dict, Generator, Optional

if TYPE_CHECKING:
    from .client import Client, RustKissVDBError


class StreamAPI:
    def __init__(self, client: Client) -> None:
        self._client = client

    def events(
        self,
        *,
        since: Optional[int] = None,
        types: Optional[str] = None,
        key_prefix: Optional[str] = None,
    ) -> Generator[Dict[str, object], None, None]:
        params = {}
        if since is not None:
            params["since"] = int(since)
        if types:
            params["types"] = types
        if key_prefix:
            params["key_prefix"] = key_prefix

        with self._client.stream_request("GET", "/v1/stream", params=params) as resp:
            if resp.status_code >= 400:
                raise RustKissVDBError(self._client._error_message(resp))
            event: Dict[str, object] = {}
            for line in resp.iter_lines():
                if not line:
                    if event:
                        yield event
                        event = {}
                    continue
                text = line.decode("utf-8")
                if text.startswith("event:"):
                    event["event"] = text.split(":", 1)[1].strip()
                elif text.startswith("id:"):
                    event["id"] = text.split(":", 1)[1].strip()
                elif text.startswith("data:"):
                    payload = text.split(":", 1)[1].strip()
                    try:
                        event["data"] = json.loads(payload)
                    except json.JSONDecodeError:
                        event["data_raw"] = payload
            if event:
                yield event



from __future__ import annotations

from typing import TYPE_CHECKING, Any, Dict, List

if TYPE_CHECKING:
    from .client import Client


class SqlAPI:
    def __init__(self, client: Client) -> None:
        self._client = client

    def query(self, sql: str, params: List[Any] | None = None) -> List[Dict[str, Any]]:
        data = self._client.request(
            "POST",
            "/v1/sql/query",
            json={"sql": sql, "params": params or []},
        )
        return data.get("rows", [])

    def execute(self, sql: str, params: List[Any] | None = None) -> int:
        data = self._client.request(
            "POST",
            "/v1/sql/exec",
            json={"sql": sql, "params": params or []},
        )
        return int(data.get("rows_affected", 0))



from __future__ import annotations

from typing import TYPE_CHECKING, Any, Dict, List, Optional

if TYPE_CHECKING:
    from .client import Client



class DocAPI:
    def __init__(self, client: Client) -> None:
        self._client = client

    def put(self, collection: str, doc_id: str, doc: Dict[str, Any]) -> Dict[str, Any]:
        return self._client.request("PUT", f"/v1/doc/{collection}/{doc_id}", json=doc)

    def get(self, collection: str, doc_id: str) -> Dict[str, Any]:
        return self._client.request("GET", f"/v1/doc/{collection}/{doc_id}")

    def delete(self, collection: str, doc_id: str) -> bool:
        data = self._client.request("DELETE", f"/v1/doc/{collection}/{doc_id}")
        return bool(data.get("deleted"))

    def find(
        self,
        collection: str,
        *,
        filter: Optional[Dict[str, Any]] = None,
        limit: int = 20,
    ) -> List[Dict[str, Any]]:
        data = self._client.request(
            "POST",
            f"/v1/doc/{collection}/find",
            json={"filter": filter or {}, "limit": int(limit)},
        )
        return data.get("documents", [])



from __future__ import annotations

from typing import TYPE_CHECKING, Any, Dict, List, Optional

if TYPE_CHECKING:
    from .client import Client


class StateAPI:
    def __init__(self, client: Client) -> None:
        self._client = client

    def list(self, prefix: str = "", limit: int = 100) -> List[Dict[str, Any]]:
        return self._client.request(
            "GET",
            "/v1/state",
            params={"prefix": prefix, "limit": int(limit)},
        )

    def get(self, key: str) -> Dict[str, Any]:
        return self._client.request("GET", f"/v1/state/{key}")

    def put(
        self,
        key: str,
        value: Any,
        *,
        ttl_ms: Optional[int] = None,
        if_revision: Optional[int] = None,
    ) -> Dict[str, Any]:
        payload: Dict[str, Any] = {"value": value}
        if ttl_ms is not None:
            payload["ttl_ms"] = int(ttl_ms)
        if if_revision is not None:
            payload["if_revision"] = int(if_revision)
        return self._client.request("PUT", f"/v1/state/{key}", json=payload)

    def delete(self, key: str) -> bool:
        data = self._client.request("DELETE", f"/v1/state/{key}")
        return bool(data.get("deleted"))

    def batch_put(self, operations: List[Dict[str, Any]]) -> Dict[str, Any]:
        return self._client.request(
            "POST",
            "/v1/state/batch_put",
            json={"operations": operations},
        )



from __future__ import annotations

from typing import TYPE_CHECKING, Any, Dict, List, Optional

if TYPE_CHECKING:
    from .client import Client


class VectorAPI:
    def __init__(self, client: Client) -> None:
        self._client = client

    def create_collection(self, collection: str, dim: int, metric: str = "cosine") -> Dict[str, Any]:
        return self._client.request(
            "POST",
            f"/v1/vector/{collection}",
            json={"dim": int(dim), "metric": metric},
        )

    def upsert(
        self,
        collection: str,
        *,
        vector_id: str,
        vector: List[float],
        meta: Optional[Dict[str, Any]] = None,
    ) -> Dict[str, Any]:
        payload = {"id": vector_id, "vector": vector}
        if meta is not None:
            payload["meta"] = meta
        return self._client.request(
            "POST",
            f"/v1/vector/{collection}/upsert",
            json=payload,
        )

    def upsert_batch(
        self,
        collection: str,
        items: List[Dict[str, Any]],
    ) -> Dict[str, Any]:
        return self._client.request(
            "POST",
            f"/v1/vector/{collection}/upsert_batch",
            json={"items": items},
        )

    def delete(self, collection: str, vector_id: str) -> bool:
        data = self._client.request(
            "POST",
            f"/v1/vector/{collection}/delete",
            json={"id": vector_id},
        )
        return bool(data.get("deleted"))

    def delete_batch(self, collection: str, ids: List[str]) -> Dict[str, Any]:
        return self._client.request(
            "POST",
            f"/v1/vector/{collection}/delete_batch",
            json={"ids": ids},
        )

    def search(
        self,
        collection: str,
        vector: List[float],
        *,
        k: int = 5,
        include_meta: bool = True,
        filters: Optional[Dict[str, Any]] = None,
    ) -> Dict[str, Any]:
        payload: Dict[str, Any] = {
            "vector": vector,
            "k": int(k),
            "include_meta": bool(include_meta),
        }
        if filters is not None:
            payload["filters"] = filters
        return self._client.request(
            "POST",
            f"/v1/vector/{collection}/search",
            json=payload,
        )



from __future__ import annotations

import os
from dataclasses import dataclass
from typing import TYPE_CHECKING, Optional

if TYPE_CHECKING:
    from .client import Client


@dataclass(frozen=True, slots=True)
class Config:
    base_url: str
    api_key: Optional[str] = None
    timeout: float = 30.0

    @classmethod
    def from_env(cls, env_path: Optional[str] = None) -> "Config":
        """
        Carga configuración desde .env (si existe) + variables de entorno.

        Variables soportadas:
          - RUSTKISS_URL o VDB_BASE_URL
          - RUSTKISS_API_KEY o API_KEY
          - RUSTKISS_TIMEOUT (segundos)
          - PORT_RUST_KISS_VDB (si no se setea base_url)
        """
        # Import local para que el SDK no dependa de python-dotenv si no lo instalaste
        try:
            from dotenv import find_dotenv, load_dotenv  # type: ignore
        except Exception:
            find_dotenv = None
            load_dotenv = None

        if load_dotenv is not None:
            if env_path == "":
                load_dotenv()  # carga env actual (sin buscar)
            else:
                # busca .env si no se pasa ruta
                path = env_path or (find_dotenv() if find_dotenv is not None else None)
                load_dotenv(path)

        base = (
            os.getenv("RUSTKISS_URL")
            or os.getenv("VDB_BASE_URL")
            or f"http://127.0.0.1:{os.getenv('PORT_RUST_KISS_VDB', '9917')}"
        ).rstrip("/")

        api_key = os.getenv("RUSTKISS_API_KEY") or os.getenv("API_KEY")
        timeout = float(os.getenv("RUSTKISS_TIMEOUT", "30"))

        return cls(base_url=base, api_key=api_key, timeout=timeout)

    def create_client(self) -> Client:
        from .client import Client
        return Client(
            base_url=self.base_url,
            api_key=self.api_key,
            timeout=self.timeout,
        )
